package org.fuseri.model.dto.lecture;

import jakarta.validation.constraints.*;
import lombok.Getter;
import lombok.Setter;
import org.fuseri.model.dto.common.DomainObjectDto;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
public class LectureDto extends DomainObjectDto {

    @Future(message = "Lecture start date and time must be in the future")
    @NotNull(message = "Lecture start date and time cannot be null")
    private LocalDateTime from;

    @Future(message = "Lecture end date and time must be in the future")
    @NotNull(message = "Lecture end date and time cannot be null")
    private LocalDateTime to;

    @NotBlank(message = "Lecture topic cannot be blank")
    @Size(max = 255, message = "Lecture topic must be no more than {max} characters")
    private String topic;

    @NotNull(message = "Lecture capacity cannot be null")
    @Min(value = 1, message = "Lecture capacity must be at least 1")
    private Integer capacity;

    @NotNull(message = "Lecture capacity cannot be null")
    private String lecturerId;

    @NotBlank(message = "Lecture courseId cannot be blank")
    private String courseId;

    @NotNull(message = "Student IDs list cannot be null")
    private List<String> studentIds;

    public LectureDto(LocalDateTime from, LocalDateTime to, String topic, Integer capacity, String lecturerId, String courseId) {
        this.from = from;
        this.to = to;
        this.topic = topic;
        this.capacity = capacity;
        this.lecturerId = lecturerId;
        this.courseId = courseId;
        this.studentIds = new ArrayList<>();
    }

    @AssertTrue(message = "Lecture start datetime must be before Lecture end datetime")
    private boolean isFromDateBeforeToDate() {
        if (from == null || to == null) return true;
        return from.isBefore(to);
    }
}
