package org.fuseri.modulecertificate.service;

import jakarta.validation.Valid;
import org.fuseri.model.dto.certificate.CertificateCreateDto;
import org.fuseri.model.dto.certificate.CertificateDto;
import org.fuseri.model.dto.common.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * This class represents a RESTful controller for Certificate module.
 * It handles incoming HTTP requests for generating and retrieving course certificates,
 * and delegates them to the appropriate service method.
 */
@RestController
@RequestMapping("/certificates")
public class CertificateController {

    @Autowired
    public CertificateController() {
    }

    /**
     * Generates certificate for specified user and course.
     *
     * @param certificateCreateDto Dto with data used for generating certificate
     * @return CertificateDto with data of generated certificate
     */
    @PostMapping("/generate")
    public CertificateDto generate(@Valid @RequestBody CertificateCreateDto certificateCreateDto) {
        return new CertificateDto();
    }

    /**
     * Retrieves previously generated certificate.
     *
     * @param id ID of certificate to be retrieved
     * @return CertificateDto with data of previously generated certificate with specified ID
     */
    @GetMapping("/find")
    public CertificateDto find(@RequestParam String id) {
        return new CertificateDto();
    }

    /**
     * Retrieves previously generated certificates for user.
     *
     * @param userId ID of user to retrieve certificates for.
     * @return List of CertificateDto objects with previously generated certificates
     * for specified User.
     */
    @GetMapping("/findForUser")
    public List<CertificateDto> findForUser(@RequestParam String userId) {
        return List.of(new CertificateDto());
    }

    /**
     * Retrieves previously generated certificate ID.
     *
     * @param userId   ID of user to retrieve certificates for.
     * @param courseId ID of course to retrieve certificates for.
     * @return List of CertificateDto objects with previously generated certificates
     * for specified User.
     */
    @GetMapping("/getId")
    public String getId(@RequestParam String userId, @RequestParam String courseId) {
        return "0";
    }

    /**
     * Retrieves previously generated certificate ID.
     *
     * @param id Id of certificate to be deleted.
     */
    @DeleteMapping("/delete")
    public void delete(@RequestParam String id) {
    }

    /**
     * Find certificates and return them in a paginated format
     *
     * @param page the page number of the certificates to retrieve
     * @return a Result object containing a list of CertificateDto objects and pagination information
     */
    @GetMapping("/findAll")
    public Result<CertificateDto> findAllCertificates(@RequestParam int page) {
        return new Result<>();
    }
}
