package org.fuseri.modulelanguageschool.lecture;

import jakarta.validation.Valid;
import org.fuseri.model.dto.course.CourseDto;
import org.fuseri.model.dto.lecture.LectureCreateDto;
import org.fuseri.model.dto.lecture.LectureDto;
import org.fuseri.model.dto.user.UserDto;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.time.Month;
import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a RESTful controller for Lecture resources.
 * It handles incoming HTTP requests related to lectures, and delegates them to the appropriate service method.
 */
@RestController
@RequestMapping("/lectures")
public class LectureController {

    private static final LocalDateTime START_DATETIME = LocalDateTime.of(2045, Month.JUNE, 30, 12, 0, 0);
    private static final LocalDateTime END_DATETIME = LocalDateTime.of(2045, Month.JUNE, 30, 14, 0, 0);
    public static final String TOPIC = "Learning how to spell deprecated";

    /**
     * Creates a new lecture resource by delegating to the LectureService's create method.
     *
     * @param lecture the LectureDto representing the lecture to be created
     * @return the LectureDto representing the newly created lecture
     */
    @PostMapping("/create")
    public LectureDto create(@Valid @RequestBody LectureCreateDto lecture) {
        return new LectureDto(START_DATETIME, END_DATETIME, TOPIC, 10, "0", "0");
    }

    /**
     * Retrieves a lecture resource by its ID.
     *
     * @param id the ID of the lecture to find
     * @return the LectureDto representing the found lecture
     */
    @GetMapping("find/{id}")
    public LectureDto find(@PathVariable String id) {
        return new LectureDto(START_DATETIME, END_DATETIME, TOPIC, 10, "0", "0");
    }

    /**
     * Retrieves a list of lectures for the corresponding course
     *
     * @param courseId the course to retrieve lectures from
     * @return the list of LectureDtos
     */
    @GetMapping("/findByCourse")
    public List<LectureDto> findByCourse(@Valid @RequestParam String courseId) {
        return new ArrayList<>();
    }

    /**
     * Updates an existing lecture resource by delegating to the LectureService's update method.
     *
     * @param lecture the CourseCreateDto representing the updated lecture
     * @return the LectureDto representing the updated lecture
     */
    @PutMapping("/update/{id}")
    public LectureDto update(@PathVariable String id, @Valid @RequestBody LectureCreateDto lecture) {
        return new LectureDto(START_DATETIME, END_DATETIME, TOPIC, 10, "0", "0");
    }

    /**
     * Deletes a lecture resource by its ID.
     *
     * @param id the ID of the lecture to delete
     */
    @DeleteMapping("/delete/{id}")
    public void delete(@PathVariable String id) {
    }


    /**
     * Adds lecturer to the existing lecture resource
     *
     * @param id       id of lecture to update
     * @param lecturer UserDto for the course lecturer
     * @return the LectureDto representing the updated lecture
     */
    @PatchMapping("/setLecturer/{id}")
    public LectureDto setLecturer(@PathVariable String id, @RequestBody UserDto lecturer) {
        return new LectureDto(START_DATETIME, END_DATETIME, TOPIC, 10, "0", "0");
    }

    /**
     * Adds student to the existing lecture resource
     *
     * @param id      id of lecture to update
     * @param student UserDto for the course student
     * @return the LectureDto representing the updated lecture
     */
    @PatchMapping("/enrol/{id}")
    public LectureDto enrol(@PathVariable String id, @RequestBody UserDto student) {
        var lecture = new LectureDto(START_DATETIME, END_DATETIME, TOPIC, 10, "0", "0");
        lecture.setStudentIds(new ArrayList<>(List.of(student.getId())));
        return lecture;
    }

    /**
     * Removes student from the existing lecture resource
     *
     * @param id      id of lecture to update
     * @param student UserDto for the course student
     * @return the LectureDto representing the updated lecture
     */
    @PatchMapping("/expel/{id}")
    public LectureDto expel(@PathVariable String id, @RequestBody UserDto student) {
        return new LectureDto(START_DATETIME, END_DATETIME, TOPIC, 10, "0", "0");
    }
}